#include "jpg.h"
#include "../mem.h"

XImage* read_jpg(Display *d, const char* filename) {
  struct jpeg_decompress_struct info;
  struct jpeg_error_mgr err;
  int depth = DefaultDepth(d, DefaultScreen(d));

  info.err = jpeg_std_error(&err);
  FILE* file = fopen(filename, "rb");
  if (file == NULL) {
    fprintf(stderr, "「%s」を開けられません。\n", filename);
    return NULL;
  }

  jpeg_create_decompress(&info);
  jpeg_stdio_src(&info, file);
  jpeg_read_header(&info, TRUE);
  info.out_color_space = JCS_RGB;
  jpeg_start_decompress(&info);

  int width = info.image_width;
  int height = info.image_height;
  int row = width * info.output_components;

  // 画像データを保存する為に、配列を創作する
  unsigned char* data = (unsigned char*)malloc(row * height);
  if (data == NULL) {
    jpeg_destroy_decompress(&info);
    fclose(file);
    return NULL;
  }

  JSAMPARRAY buf = (*info.mem->alloc_sarray) ((j_common_ptr)&info, JPOOL_IMAGE, row, 1);
  if (buf == NULL) {
    fprintf(stderr, "JPGデータを創作に失敗しました。\n");
    jpeg_destroy_decompress(&info);
    fclose(file);
    free(data);
    return NULL;
  }

  while (info.output_scanline < info.output_height) {
    unsigned char* dest = &data[info.output_scanline * row];
    jpeg_read_scanlines(&info, buf, 1);
    smemcpy(dest, row, buf[0], row);
  }

  // 掃除
  jpeg_finish_decompress(&info);
  jpeg_destroy_decompress(&info);
  fclose(file);

  // XImageを創作
  int bitmap_order = (ImageByteOrder(d) == LSBFirst) ? LSBFirst : MSBFirst;
  int byte_order = (BitmapBitOrder(d) == LSBFirst) ? LSBFirst : MSBFirst;
  XImage* img = XCreateImage(d, CopyFromParent, depth, ZPixmap, 0, (char*)data, width, height, 32, row);
  if (img == NULL) {
    fprintf(stderr, "JPGのXImageを創作に失敗しました。\n");
    free(data);
    return NULL;
  }

  img->bitmap_bit_order = bitmap_order;
  img->byte_order = byte_order;

  return img;
}
